//グラフィックス
#ifndef HEADER_GRAPHICS_H
#define HEADER_GRAPHICS_H
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <GL/glut.h>
#include "ofl.h"
#include "macro.h"
using namespace nsOfl;
static const uint32_t OFL_MODEL_MAX=16;
enum{
	MODEL1=0,
	MODEL_MAX,
};

void FinalizeGraphics(void);


class Geometry
{
	public:
		Geometry();
		virtual ~Geometry();

		void		Init( void );
		void		Clean( void );

		oflModel_t*	GetModel( uint32_t val );
		oflModel_t*	GetModel( void );
		void		SetModel( uint32_t index, char *filename );

		void		SetModelName( uint32_t index, char *name );
		char*		GetModelName( uint32_t index );
		char*		GetModelName( );
		
		void		SetModelIndex( uint32_t val );
		uint32_t 	GetModelIndex( void );

		void		SetMaterialMode( uint32_t val );
		uint32_t 	GetMaterialMode( void );
		void		IncreMaterialMode( void );
		void		DecreMaterialMode( void );
		void		SetScale( double val );
		double		GetScale( void );
		void		SetSmoothingAngle( double val );
		double		GetSmoothingAngle( void );
		void		SetWeldDistance( double val );
		double		GetWeldDistance( void );

	private:
		float			m_scale;						// original scale factor
		float			m_smoothing_angle;			// smoothing angle
		double			m_weld_distance;			// epsilon for welding vertices

		uint32_t		m_material_mode;			/// 0=none, 1=color, 2=material
		uint32_t		m_model_index;
		char			**m_model_filename;
		//char			*cGeometryFile2;
		oflModel_t		**oflModel;					// ofl model data structure 
		//char			*g_cModelFile = NULL;		// name of the obect file 

	protected:

};

class Window
{
	public:
		Window();
		virtual ~Window();

		void		Init(void);
		void		Clean(void);

		uint32_t	GetWindowWidth(void);
		void		SetWindowWidth(uint32_t val);
		uint32_t	GetWindowHeight(void);
		void		SetWindowHeight(uint32_t val);

		uint32_t	GetWindowX(void );
		void		SetWindowX(uint32_t val);
		uint32_t	GetWindowY(void);
		void		SetWindowY(uint32_t val);

	private:
		uint32_t	m_window_width;
		uint32_t	m_window_height;
		uint32_t	m_window_x;
		uint32_t	m_window_y;

	protected:

};

class Graphics
{
	public:
		Graphics();
		virtual ~Graphics();
		
		Geometry	m_Geom;
		Window		m_Window;

		void		Init(void);
		void		CleanGraphicsState( void );

		void		GenerateLists( uint32_t target );
		int32_t		GetModelList( void );
		void		SetModelList( int32_t val );

		uint32_t	GetMenuEntries( void );
		void		SetMenuEntries( uint32_t entries );

		bool		GetDrawFacetNormalEnable( void );
		bool		GetDispBoundingBoxEnable( void );
		bool		GetDispStatusEnable( void );
		bool		GetDispPerformanceEnable( void );

	private:
		int			m_mode_list;				// display list for object
		bool		m_draw_facet_normal_enable;	// draw with facet normal?
		bool		m_disp_status_enable;		/// 状態表示
		bool		m_disp_bounding_box_enable;	/// bounding box on?
		bool		m_disp_performance_enable;	/// performance counter on?
		int			m_menu_entries;					// g_gs.iEntries in model menu
	protected:
};

#endif // HEADER

